/***************************************************************************
 * This file is part of the project AqFoxExt.                              *
 * AqFoxExt (c) by 2011 Martin Preuss, all rights reserved.                *
 *                                                                         *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef FXXACTION_H
#define FXXACTION_H


#include <fx.h>


class FXXIconSet;
class FXXAction;

class FXXActionMenuCommand;
class FXXActionToolButton;
class FXXMenuContainer;
class FXXActionMenuCheck;
class FXXActionCheckButton;


#include <aqfoxext/aqfoxext.h>
#include <aqfoxext/fxx_menucontainer.hpp>



class AQFOXEXT_API FXXAction: FXObject {
  friend class FXXActionMenuCommand;
  friend class FXXActionToolButton;
  friend class FXXActionCheckButton;
  friend class FXXMenuContainer;
  friend class FXXActionMenuCheck;
protected:
  FXObjectListOf<FXXActionMenuCommand> commandList;
  FXObjectListOf<FXXActionToolButton> toolButtonList;
  FXObjectListOf<FXXMenuContainer> containerList;
  FXObjectListOf<FXXActionMenuCheck> menuCheckList;
  FXObjectListOf<FXXActionCheckButton> checkButtonList;
  FXString name;
  FXString text;
  FXString helpText;
  FXString tipText;
  FXXIconSet *iconSet;
  FXObject *target;
  FXSelector selector;
  FXbool m_check;

  FXXAction():FXObject(){};

  void removeMenuCommand(FXXActionMenuCommand *w);
  void removeMenuCheck(FXXActionMenuCheck *w);
  void removeToolButton(FXXActionToolButton *w);
  void removeCheckButton(FXXActionCheckButton *w);
  void removeMenuContainer(FXXMenuContainer *w);

  void removeFrom(FXWindow *p);

public:
  FXXAction(const FXString &name,
	    const FXString &text,
	    FXXIconSet *iconSet,
	    FXObject *tgt=NULL,
	    FXSelector sel=0);
  virtual ~FXXAction();

  virtual void enable();
  virtual void disable();

  void setCheck(FXbool s=TRUE);

  /// Get check state (TRUE, FALSE or MAYBE)
  FXbool getCheck() const { return m_check; }

  void addToMenu(FXMenuPane* p,
                 FXuint opts=0,
                 FXbool withText=TRUE, FXbool withIcon=FALSE);
  void addToMenu(FXXMenuContainer *p,
                 FXuint opts=0,
                 FXbool withText=TRUE, FXbool withIcon=FALSE);
  void addToMenuCheckable(FXMenuPane* p, FXuint opts);

  void addToToolBar(FXToolBar *p,
                    FXuint opts=0,
                    FXbool withText=TRUE, FXbool withIcon=FALSE);
  void addToToolBarCheckable(FXToolBar *p, FXuint opts=0);

  void removeFromAll();

  /// Set the status line help text for this label
  void setHelpText(const FXString& text){ helpText=text; }

  /// Get the status line help text for this label
  const FXString& getHelpText() const { return helpText; }

  /// Set the tool tip message for this label
  void setTipText(const FXString& text){ tipText=text; }

  /// Get the tool tip message for this label
  const FXString& getTipText() const { return tipText; }

  const FXString &getName() const { return name;};
  const FXString &getText() const { return text;};
  FXXIconSet *getIconSet() const { return iconSet;};
  FXObject *getTarget() const { return target;};
  FXSelector getSelector() const { return selector;};

  void create();
};



#endif

