/*
FoxTray, system tray icon for the FOX Toolkit

Copyright (c) 2008, Hendrik Rittich
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are
met:

    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in 
      the documentation and/or other materials provided with the 
      distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef FX_TRAY_ICON_H
#define FX_TRAY_ICON_H

#include "fx.h"

#include <aqfoxext/aqfoxext.h>

namespace FX {

/** @defgroup tray_options FXXTrayIcon options 
  @{*/

/** FXXTrayIcon options.
 Describe what to do on mouse interaction with the icon.
 */
enum {
    TRAY_MENU_ON_LEFT = 0x0001, /**< Open popup menu on mouse left click. */
    TRAY_MENU_ON_RIGHT = 0x0002, /**< Open popup menu on mouse right click. */
    TRAY_CMD_ON_LEFT = 0x0004, /**< Send SEL_COMMAND message on mouse left click. */
    TRAY_CMD_ON_RIGHT = 0x0008 /**< Send SEL_COMMAND message on mouse right click. */
};
/** @} */

/** Display an icon in the system tray.
 Display an icon in the system tray, also
 called notification area.
 */
class AQFOXEXT_API FXXTrayIcon : public FXTopWindow
{
    FXDECLARE(FXXTrayIcon)
public: 
    /** @cond DEV */
    enum {
        ID_BUTTON = FXTopWindow::ID_LAST,
        ID_LAST
    };
    /** @endcond */

    /** Create a tray icon.
     @param app the current FXApp instance
     @param text the tool tip text
     @param icon the icon to display in the tray
     @param pup the popup menu to display
     @param target send messages to
     @param sel id 
     @param opts options. See @ref tray_options .
     */
    FXXTrayIcon(FXApp* app, const FXString& text, FXIcon* icon, 
            FXPopup* pup = NULL, FXObject* target = 0, FXSelector sel = 0,
            FXuint opts = TRAY_MENU_ON_LEFT);

    ~FXXTrayIcon(); 

    /** Create the server side resource of the tray icon. */
    void create();

    /** @cond DEV */
    void mapToManager(); 

    long onLeft(FXObject*, FXSelector, void*);
    long onRight(FXObject* obj, FXSelector, void* ptr);
    long onClose(FXObject*, FXSelector, void*);
    /** @endcond */

    /** Set the popup menu. */
    void setMenu(FXPopup* pup) { mPup = pup; }
    /** Get the popup menu. */
    FXPopup* getMenu() const { return mPup; }

    /** Change the icon in the tray. */
    void setIcon(FXIcon* icon);
    /** Get the icon. */
    FXIcon* getIcon() const;

    /** Change the tool tip text. */
    void setText(const FXString& text);
    /** Get the tool tip text. */
    const FXString& getText() const;

private:
    FXPopup* mPup;
    FXuint mOpts;
    FXButton* btn;

    void popup(FXint x, FXint y);
    FXXTrayIcon() {}
};

/** @example TrayApp.cpp
    A small example Application.
*/

}

#endif

