/***************************************************************************
 * This file is part of the project AqFoxExt.                              *
 * AqFoxExt (c) by 2011 Martin Preuss, all rights reserved.                *
 *                                                                         *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "fxx_iconset.hpp"
#include "fxx_app.hpp"




FXXIconSet::FXXIconSet(const FXIcon *ic)
:m_iconOrig(NULL)
,m_iconSmall(NULL)
,m_iconMedium(NULL)
,m_iconBig(NULL)
{
  m_iconOrig=copyIcon(ic);
  m_iconOrig->create();
  refreshIcons();
}



FXXIconSet::~FXXIconSet() {
}




FXIcon *FXXIconSet::copyIcon(const FXIcon *ic) {
  if (ic==NULL)
    return NULL;
  else {
    FXint w, h, len;
    FXColor *ptr;
    FXIcon *t;
    FXApp *a;
  
    a=ic->getApp();
    w=ic->getWidth();
    h=ic->getHeight();
    len=w*h;
    if (!FXMALLOC(&ptr, FXColor, len)) {
      return NULL;
    }
    memmove(ptr, ic->getData(), len*sizeof(FXColor));
    t=new FXIcon(a, NULL,
                 ic->getTransparentColor(),
                 IMAGE_DITHER|IMAGE_KEEP|IMAGE_SHMI|IMAGE_SHMP,
                 w, h);
    t->setData(ptr, IMAGE_OWNED);
    return t;
  }
}



void FXXIconSet::refreshIcons() {
  if (m_iconOrig) {
    FXIcon *t;
    FXXApp *a;
  
    a=dynamic_cast<FXXApp*>(m_iconOrig->getApp());

    if (m_iconSmall) {
      delete m_iconSmall;
      m_iconSmall=NULL;
    }
    if (m_iconMedium) {
      delete m_iconMedium;
      m_iconMedium=NULL;
    }
    if (m_iconBig) {
      delete m_iconBig;
      m_iconBig=NULL;
    }
  
    // small icon
    t=copyIcon(m_iconOrig);
    if (t) {
      t->scale(a->getSizeSmallIcons().w,
               a->getSizeSmallIcons().h);
      t->create();
      m_iconSmall=t;
    }
  
    // medium icon
    t=copyIcon(m_iconOrig);
    if (t) {
      t->scale(a->getSizeMediumIcons().w,
               a->getSizeMediumIcons().h);
      t->create();
      m_iconMedium=t;
    }
  
    // big icon
    t=copyIcon(m_iconOrig);
    if (t) {
      t->scale(a->getSizeBigIcons().w,
               a->getSizeBigIcons().h);
      t->create();
      m_iconBig=t;
    }
  }
}



