/***************************************************************************
 * This file is part of the project AqFoxExt.                              *
 * AqFoxExt (c) by 2011 Martin Preuss, all rights reserved.                *
 *                                                                         *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "fxx_checktree.hpp"
#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>


#define SIDE_SPACING        4   // Spacing between side and item
#define ICON_SPACING        4   // Spacing between parent and child in x direction

#define SELECT_MASK         (FOLDINGLIST_SINGLESELECT|FOLDINGLIST_BROWSESELECT)




FXXCheckTreeItem::FXXCheckTreeItem(const FXString& text, FXIcon* oi, FXIcon* ci, void* ptr)
:FXFoldingItem(text, oi, ci, ptr)
,m_isChecked(FALSE)
,m_useOwnColors(false)
{
}



FXXCheckTreeItem::~FXXCheckTreeItem() {
}



FXString FXXCheckTreeItem::getColumnTooltipText(FXint col) {
  return getText().section('\t', 0);
}



FXString FXXCheckTreeItem::getItemText(const FXXCheckTree* list) const {
  return getText();
}




void FXXCheckTreeItem::draw(const FXFoldingList* list,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const {
  const FXXCheckTree *ctree=dynamic_cast<const FXXCheckTree*>(list);

  if (ctree) {
    register FXHeader *header=list->getHeader();
    register FXIcon *icon=(state&OPENED)?openIcon:closedIcon;
    register FXint ih=0, iw=0, xb, beg, end, hi, drw, space, used, xx;
    register FXint ix, iy;
    FXString itemText=getItemText(ctree);

    if(header->getNumItems()==0) return;
    xx=x+SIDE_SPACING/2;
    if(icon){
      iw=icon->getWidth();
      ih=icon->getHeight();
      dc.setClipRectangle(header->getItemOffset(0),y,header->getItemSize(0),h);
      dc.drawIcon(icon,xx,y+(h-ih)/2);
      dc.clearClipRectangle();
      xx+=ICON_SPACING+iw;
    }

    if (ctree->getOptions() & FXXCheckTree::OPTIONS_CHECKMARKS) {
      /* draw checkbox */
      ix=xx;
      iy=y+(h-10)/2;
      if (m_useOwnColors)
	dc.setForeground(m_backColor);
      else
	dc.setForeground(list->getBackColor());
      dc.fillRectangle(ix+2,iy+2,9,9);

      dc.setForeground(ctree->shadowColor);
      dc.fillRectangle(ix,iy,12,1);
      dc.fillRectangle(ix,iy,1,12);
      dc.setForeground(ctree->borderColor);
      dc.fillRectangle(ix+1,iy+1,10,1);
      dc.fillRectangle(ix+1,iy+1,1,10);
      dc.setForeground(ctree->hiliteColor);
      dc.fillRectangle(ix,iy+12,13,1);
      dc.fillRectangle(ix+12,iy,1,13);
      dc.setForeground(ctree->baseColor);
      dc.fillRectangle(ix+1,iy+11,11,1);
      dc.fillRectangle(ix+11,iy+1,1,11);

      if (!isEnabled()) {
	if (m_useOwnColors)
	  dc.setForeground(makeShadowColor(m_backColor));
	else
	  dc.setForeground(makeShadowColor(list->getBackColor()));
      }
      else {
	if (m_useOwnColors)
	  dc.setForeground(m_textColor);
	else
	  dc.setForeground(list->getTextColor());
      }

      if(m_isChecked!=FALSE){
	FXSegment seg[6];

#ifndef WIN32
	seg[0].x1=3+ix; seg[0].y1=5+iy; seg[0].x2=5+ix; seg[0].y2=7+iy;
	seg[1].x1=3+ix; seg[1].y1=6+iy; seg[1].x2=5+ix; seg[1].y2=8+iy;
	seg[2].x1=3+ix; seg[2].y1=7+iy; seg[2].x2=5+ix; seg[2].y2=9+iy;
	seg[3].x1=5+ix; seg[3].y1=7+iy; seg[3].x2=9+ix; seg[3].y2=3+iy;
	seg[4].x1=5+ix; seg[4].y1=8+iy; seg[4].x2=9+ix; seg[4].y2=4+iy;
	seg[5].x1=5+ix; seg[5].y1=9+iy; seg[5].x2=9+ix; seg[5].y2=5+iy;
#else
	seg[0].x1=3+ix; seg[0].y1=5+iy; seg[0].x2=5+ix; seg[0].y2=7+iy;
	seg[1].x1=3+ix; seg[1].y1=6+iy; seg[1].x2=5+ix; seg[1].y2=8+iy;
	seg[2].x1=3+ix; seg[2].y1=7+iy; seg[2].x2=5+ix; seg[2].y2=9+iy;
	seg[3].x1=5+ix; seg[3].y1=7+iy; seg[3].x2=10+ix; seg[3].y2=2+iy;
	seg[4].x1=5+ix; seg[4].y1=8+iy; seg[4].x2=10+ix; seg[4].y2=3+iy;
	seg[5].x1=5+ix; seg[5].y1=9+iy; seg[5].x2=10+ix; seg[5].y2=4+iy;
#endif
	dc.drawLineSegments(seg,6);
      }
      xx+=12;
    }

    if(!itemText.empty()){
      xb=header->getItemOffset(0)+header->getItemSize(0);
      if(xb>xx)
	xb=xx;
      if (isSelected()){
	dc.setForeground(list->getSelBackColor());
      }
      else {
	if (m_useOwnColors)
	  dc.setForeground(m_backColor);
	else
	  dc.setForeground(list->getBackColor());
      }
      dc.fillRectangle(xb,y,header->getTotalSize()-xb,h);

      if (hasFocus())
	dc.drawFocusRectangle(xb+1,y+1,header->getTotalSize()-xb-2,h-2);

      used=xx-header->getItemOffset(0);
      for (hi=beg=0; beg<itemText.length() && hi<header->getNumItems(); hi++,beg=end+1){
	space=header->getItemSize(hi)-used;
	for (end=beg; end<itemText.length() && itemText[end]!='\t'; end++);
	if (end>beg){
	  drw=end-beg;
	  drawColumn(ctree, dc, hi, xx, y, space, h, &itemText[beg], drw);
	}
	xx+=space;
	used=0;
      }
    }
  }
  else
    FXFoldingItem::draw(list, dc, x, y, w, h);
}



void FXXCheckTreeItem::drawColumn(const FXXCheckTree* list, FXDC &dc, FXint col,
				  FXint xx, FXint y, FXint w, FXint h,
				  const FXchar* string, FXuint length) const {
  register FXFont *font=list->getFont();
  register FXint tw=font->getTextWidth(string, length);
  register FXint th=font->getFontHeight();
  register FXint yt=y+(h-th-4)/2;

  if (!isEnabled()) {
    if (m_useOwnColors)
      dc.setForeground(makeShadowColor(m_backColor));
    else
      dc.setForeground(makeShadowColor(list->getBackColor()));
  }
  else if (isSelected())
    dc.setForeground(list->getSelTextColor());
  else {
    if (m_useOwnColors)
      dc.setForeground(m_textColor);
    else
      dc.setForeground(list->getTextColor());
  }

  if(tw>w-4){
    FXint dw=font->getTextWidth("...",3);

    while((tw=font->getTextWidth(string, length))+dw>w-4 && length>1)
      length--;
    dc.setClipRectangle(xx, y, w, h);
    dc.drawText(xx+2, yt+font->getFontAscent()+2, string, length);
    dc.drawText(xx+tw+2, yt+font->getFontAscent()+2, "...", 3);
    dc.clearClipRectangle();
  }
  else
    dc.drawText(xx+2,yt+font->getFontAscent()+2, string, length);
}



void FXXCheckTreeItem::drawColumnRightJustified(const FXXCheckTree* list, FXDC &dc, FXint col,
                                                FXint xx, FXint y, FXint w, FXint h,
                                                const FXchar* string, FXuint length) const {
  register FXFont *font=list->getFont();
  register FXint tw=font->getTextWidth(string, length);
  register FXint th=font->getFontHeight();
  register FXint yt=y+(h-th-4)/2;
  
  if (!isEnabled())
    dc.setForeground(makeShadowColor(list->getBackColor()));
  else if(isSelected())
    dc.setForeground(list->getSelTextColor());
  else {
    if (string[0]=='-')
      dc.setForeground(FXRGB(255, 0, 0));
    else
      dc.setForeground(list->getTextColor());
  }
  
  if(tw>w-4){
    FXint dw=font->getTextWidth("...",3);
  
    while((tw=font->getTextWidth(string, length))+dw>w-4 && length>1)
      length--;
    dc.setClipRectangle(xx, y, w, h);
    dc.drawText(xx+(w-tw)-2, yt+font->getFontAscent()+2, string, length);
    dc.drawText(xx+tw+(w-tw)-2, yt+font->getFontAscent()+2, "...", 3);
    dc.clearClipRectangle();
  }
  else
    dc.drawText(xx+(w-tw)-2, yt+font->getFontAscent()+2, string, length);
}



FXint FXXCheckTreeItem::hitItem(const FXFoldingList* list, FXint x, FXint y) const {
  const FXXCheckTree *ctree=dynamic_cast<const FXXCheckTree*>(list);
  register FXint oiw=0,ciw=0,oih=0,cih=0,tw=0,th=0,iw,ih,ix,iy,tx,ty,h;
  register FXFont *font=list->getFont();
  if (openIcon){
    oiw=openIcon->getWidth();
    oih=openIcon->getHeight();
  }
  if (closedIcon){
    ciw=closedIcon->getWidth();
    cih=closedIcon->getHeight();
  }
  if (!label.empty()){
    if(!list->getHeader()->getNumItems())
      tw=4+font->getTextWidth(label.text(),label.length());
    else
      tw=4+list->getHeader()->getDefaultWidth();
  }
  th=4+getHeight(list);

  iw=FXMAX(oiw,ciw);
  ih=FXMAX(oih,cih);
  h=FXMAX(th,ih);
  ix=SIDE_SPACING/2;
  tx=SIDE_SPACING/2;
  if(iw) tx+=iw+ICON_SPACING;
  iy=(h-ih)/2;
  ty=(h-th)/2;

#if 0
  DBG_ERROR(0,
            "HitItem: x=%d, y=%d, ix=%d, iy=%d, iw=%d, ih=%d, tx=%d, ty=%d, tw=%d, th=%d",
            x, y, ix, iy, iw, ih, tx, ty, tw, th);
#endif

  // In icon?
  if (ix<=x && iy<=y && x<ix+iw && (y<iy+ih))
    return 1;

  // In text?
  if ((ctree->getOptions() & FXXCheckTree::OPTIONS_CHECKMARKS) && x<tx+12)
    return 4;

  if (tx<=x && ty<=y && /* x<tx+tw &&*/ y<ty+th) {
    return 2;
  }

  // Outside
  return 0;
}








FXDEFMAP(FXXCheckTree) FXXCheckTreeMap[]={
  FXMAPFUNC(SEL_LEFTBUTTONPRESS, 0, FXXCheckTree::onLeftBtnPress),
  FXMAPFUNC(SEL_QUERY_TIP, 0, FXXCheckTree::onQueryTip),
  FXMAPFUNC(SEL_COMMAND, FXFoldingList::ID_HEADER_CHANGE, FXXCheckTree::onHeaderClicked)
};


// Object implementation
FXIMPLEMENT(FXXCheckTree, FXFoldingList, FXXCheckTreeMap, ARRAYNUMBER(FXXCheckTreeMap))




FXXCheckTree::FXXCheckTree(FXComposite *p,
			   FXObject* tgt, FXSelector sel,
			   FXuint opts,
			   FXint x, FXint y, FXint w, FXint h)
:FXFoldingList(p, tgt, sel, opts, x, y, w, h)
{
  baseColor=getApp()->getBaseColor();
  hiliteColor=getApp()->getHiliteColor();
  shadowColor=getApp()->getShadowColor();
  borderColor=getApp()->getBorderColor();
}



FXXCheckTree::FXXCheckTree()
:FXFoldingList()
{
}



FXXCheckTree::~FXXCheckTree() {
}



void FXXCheckTree::setHeaderArrowDir(int c, FXbool b) {
  header->setArrowDir(c, b);
}



FXbool FXXCheckTree::getHeaderArrowDir(int c) const {
  return header->getArrowDir(c);
}



void FXXCheckTree::sort() {
  int i;

  for (i=0; i<getNumHeaders(); i++) {
    FXbool b;

    b=getHeaderArrowDir(i);
    if (b!=MAYBE) {
      sortByColumn(i, b);
      break;
    }
  }
}



void FXXCheckTree::sortByColumn(int c, bool up) {
}



long FXXCheckTree::onHeaderClicked(FXObject*, FXSelector, void* ptr) {
  FXint idx=(FXint)(FXival)ptr;
  FXbool b;
  int i;

  b=header->getArrowDir(idx);
  for (i=0; i<header->getNumItems(); i++)
    header->setArrowDir(i, MAYBE);

  if (b==TRUE) {
    /* was up, becomes down */
    header->setArrowDir(idx, FALSE);
    sortByColumn(idx, false);
  }
  else if (b==FALSE) {
    /* was down, becomes MAYBE */
    header->setArrowDir(idx, MAYBE);
  }
  else {
    /* was MAYBE, becomes up */
    header->setArrowDir(idx, TRUE);
    sortByColumn(idx, true);
  }
  return 1;
}



void FXXCheckTree::appendHeader(const FXString& text, FXIcon *icon, FXint size) {
  THEMEHEADERITEM *item=new THEMEHEADERITEM(text, icon, size);

  header->appendItem(item);
}



long FXXCheckTree::onQueryTip(FXObject* sender,FXSelector sel,void* ptr){
  if (FXWindow::onQueryTip(sender,sel,ptr))
    return 1;
  if((flags&FLAG_TIP) && !(options&FOLDINGLIST_AUTOSELECT)){   // No tip when autoselect!
    FXint x,y;
    FXuint buttons;

    getCursorPosition(x, y, buttons);
    FXFoldingItem *item=getItemAt(x,y);
    if (item) {
      FXXCheckTreeItem *ci;

      ci=dynamic_cast<FXXCheckTreeItem*>(item);
      if (ci) {
	int i;

	for (i=0; i<header->getNumItems(); i++) {
	  if ((x>=header->getItemOffset(i)) &&
	      (x<(header->getItemOffset(i)+header->getItemSize(i)))) {
	    FXString str=ci->getColumnTooltipText(i);
	    if (!str.empty())
	      sender->handle(this, FXSEL(SEL_COMMAND, ID_SETSTRINGVALUE), (void*)&str);
	    return 1;
	  }
	}
      }
    }
  }
  return 0;
}



