/***************************************************************************
 * This file is part of the project AqFoxExt.                              *
 * AqFoxExt (c) by 2011 Martin Preuss, all rights reserved.                *
 *                                                                         *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "fxx_htmltreeitem.hpp"

#include <gwen-gui-fox16/fox16_htmlctx.hpp>




FXXHtmlTreeItem::FXXHtmlTreeItem(const FXString& text, FXIcon* oi, FXIcon* ci, void* ptr)
:FXXCheckTreeItem(text, oi, ci, ptr)
{

}



FXXHtmlTreeItem::~FXXHtmlTreeItem() {
}



void FXXHtmlTreeItem::drawColumn(const FXXCheckTree* list,
				 FXDC &dc,
				 FXint col,
				 FXint x, FXint y, FXint w, FXint h,
				 const FXchar* str, FXuint length) const {
  if (col==0)
    return FXXCheckTreeItem::drawColumn(list, dc, col, x, y, w, h, str, length);
  else {
    FOX16_HtmlCtx html(0);
    FXString t;
    int py;
  
    t=FXString(str, length);
    if (!isEnabled()) {
      if (m_useOwnColors)
	html.setForegroundColor(makeShadowColor(m_backColor));
      else
	html.setForegroundColor(makeShadowColor(list->getBackColor()));
    }
    else if (isSelected())
      html.setForegroundColor(list->getSelTextColor());
    else {
      if (m_useOwnColors)
	html.setForegroundColor(m_textColor);
      else
	html.setForegroundColor(list->getTextColor());
    }
  
    html.setText(t.text());
    html.layout(w, h-2);
    py=y+(h-html.getHeight())/2;
  
    dc.setClipRectangle(x, y, w, h);
    html.paint(&dc, x, py+2);
    dc.clearClipRectangle();
  }
}



FXint FXXHtmlTreeItem::getHeight(const FXFoldingList* list) const {
  const FXXCheckTree *tree;

  tree=dynamic_cast<const FXXCheckTree*>(list);
  if (tree) {
    FXHeader *header=list->getHeader();
    FXString str;
    FXint n=0;
    int h;
    int cols;

    str=getItemText(tree);
    cols=str.contains('\t');
    h=FXXCheckTreeItem::getHeight(list);
    for (n=0; n<cols; n++) {
      FXString t=str.section('\t', n);
      int hc;

      if (n>0) {
	FOX16_HtmlCtx html(0);

	html.setText(t.text());
	html.layout(header->getItemSize(n), -1);
	hc=html.getHeight();
      }
      else {
	hc=list->getFont()->getFontHeight();
      }
      if (hc>h)
	h=hc;
    }
    return h+2;
  }
  return FXXCheckTreeItem::getHeight(list);
}













